package com.smppw.modaq.application.components;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.smppw.modaq.common.conts.EmailTypeConst;
import com.smppw.modaq.common.enums.ReportType;
import com.smppw.modaq.common.exception.ReportParseException;
import com.smppw.modaq.domain.dto.report.ReportAssetAllocationDTO;
import jakarta.mail.internet.MimeUtility;

import java.io.IOException;
import java.time.YearMonth;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


public final class ReportParseUtils {
    /**
     * 行业配置的表格列名称
     */
    public static final List<String> INDUSTRY_COLUMN_NAMES = ListUtil.list(false);
    /**
     * 份额变动的表格列名称
     */
    public static final List<String> SHARE_CHANGE_COLUMN_NAMES = ListUtil.list(false);
    /**
     * 主要财务指标识别列名称
     */
    public static final List<String> FINANCIAL_INDICATORS_COLUMN_NAMES = ListUtil.list(false);
    /**
     * 资产配置明细和大类关系映射
     */
    public static final Map<String, String> ASSET_ALLOCATION_TYPE_MAPPER = MapUtil.newHashMap(32, true);

    // 预编译所有正则表达式（线程安全）
    private static final Map<Pattern, Function<Matcher, String>> PATTERNS = new LinkedHashMap<>() {{
        // 季度报告（最高优先级）
        put(Pattern.compile("(20\\d{2})[^\\d]*([一二三四1234])季"), matcher -> {
            String year = matcher.group(1);
            return switch (matcher.group(2)) {
                case "一", "1" -> year + "-03-31";
                case "二", "2" -> year + "-06-30";
                case "三", "3" -> year + "-09-30";
                case "四", "4" -> year + "-12-31";
                default -> null;
            };
        });

        // 明确日期格式（次优先级）
        put(Pattern.compile("\\d{4}-\\d{2}-\\d{2}"), Matcher::group);

        // 年度报告（合并相似正则）
        put(Pattern.compile("(20\\d{2})年(度|年度)"), matcher -> matcher.group(1) + "-12-31");

        // 月份相关格式（统一处理）
        put(Pattern.compile("(20\\d{2})年[_-]?(\\d{1,2})月"), matcher -> {
            int year = Integer.parseInt(matcher.group(1));
            int month = Integer.parseInt(matcher.group(2));
            return formatMonthEnd(year, month);
        });

        // 紧凑格式（如202312月）
        put(Pattern.compile("(\\d{4})(\\d{2})月"), matcher -> {
            int year = Integer.parseInt(matcher.group(1));
            int month = Integer.parseInt(matcher.group(2));
            return formatMonthEnd(year, month);
        });

        // 纯数字格式（最低优先级）
        put(Pattern.compile("(?<!\\d)\\d{4}(0[1-9]|1[0-2])(?!\\d)"), matcher -> {
            String group = matcher.group();
            int year = Integer.parseInt(group.substring(0, 4));
            int month = Integer.parseInt(group.substring(4, 6));
            return formatMonthEnd(year, month);
        });
        put(Pattern.compile("(?<!\\d)\\d{4}(0[1-9]|1[0-2])(0[1-9]|[12]\\d|3[01])(?!\\d)"), Matcher::group);
    }};


    static {
        // 财务指标
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("期末基金净资产");
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("期末基金资产净值");

        FINANCIAL_INDICATORS_COLUMN_NAMES.add("报告期期末单位净值");
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("期末基金份额净值");

        FINANCIAL_INDICATORS_COLUMN_NAMES.add("本期利润");
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("本期已实现收益");
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("期末可供分配利润");
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("期末可供分配基金份额利润");
        FINANCIAL_INDICATORS_COLUMN_NAMES.add("基金份额累计净值增长率");

        // 中国证监会行业标准
        INDUSTRY_COLUMN_NAMES.add("农、林、牧、渔业");
        INDUSTRY_COLUMN_NAMES.add("采矿业");
        INDUSTRY_COLUMN_NAMES.add("制造业");
        INDUSTRY_COLUMN_NAMES.add("电力、热力、燃气及水生产和供应业");
        INDUSTRY_COLUMN_NAMES.add("建筑业");
        INDUSTRY_COLUMN_NAMES.add("批发和零售业");
        INDUSTRY_COLUMN_NAMES.add("交通运输、仓储和邮政业");
        INDUSTRY_COLUMN_NAMES.add("住宿和餐饮业");
        INDUSTRY_COLUMN_NAMES.add("信息传输、软件和信息技术服务业");
        INDUSTRY_COLUMN_NAMES.add("金融业");
        INDUSTRY_COLUMN_NAMES.add("房地产业");
        INDUSTRY_COLUMN_NAMES.add("租赁和商务服务业");
        INDUSTRY_COLUMN_NAMES.add("科学研究和技术服务业");
        INDUSTRY_COLUMN_NAMES.add("水利、环境和公共设施管理业");
        INDUSTRY_COLUMN_NAMES.add("居民服务、修理和其他服务业");
        INDUSTRY_COLUMN_NAMES.add("教育");
        INDUSTRY_COLUMN_NAMES.add("卫生和社会工作");
        INDUSTRY_COLUMN_NAMES.add("文化、体育和娱乐业");
        INDUSTRY_COLUMN_NAMES.add("综合");

        INDUSTRY_COLUMN_NAMES.add("港股通");

        // 以下为国际标准
        INDUSTRY_COLUMN_NAMES.add("能源");
        INDUSTRY_COLUMN_NAMES.add("原材料");
        INDUSTRY_COLUMN_NAMES.add("材料");
        INDUSTRY_COLUMN_NAMES.add("工业");
        INDUSTRY_COLUMN_NAMES.add("可选消费品");
        INDUSTRY_COLUMN_NAMES.add("非日常生活消费品");
        INDUSTRY_COLUMN_NAMES.add("必须消费品");
        INDUSTRY_COLUMN_NAMES.add("日常消费品");
        INDUSTRY_COLUMN_NAMES.add("医疗保健");
        INDUSTRY_COLUMN_NAMES.add("金融");
        INDUSTRY_COLUMN_NAMES.add("信息技术");
        INDUSTRY_COLUMN_NAMES.add("通讯服务");
        INDUSTRY_COLUMN_NAMES.add("电信服务");
        INDUSTRY_COLUMN_NAMES.add("公用事业");
        INDUSTRY_COLUMN_NAMES.add("房地产");

        // 份额变动表格识别列
        SHARE_CHANGE_COLUMN_NAMES.add("报告期期初基金份额总额");
        SHARE_CHANGE_COLUMN_NAMES.add("减:报告期期间基金总赎回份额");
        SHARE_CHANGE_COLUMN_NAMES.add("期末基金总份额/期末基金实缴总额");
        SHARE_CHANGE_COLUMN_NAMES.add("报告期期间基金拆分变动份额");
        SHARE_CHANGE_COLUMN_NAMES.add("报告期期间基金总申购份额");

        // 资产配置
        ASSET_ALLOCATION_TYPE_MAPPER.put("银行存款", "现金类资产");
        // 境内未上市、未挂牌公司股权投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("股权投资", "境内未上市、未挂牌公司股权投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其中:优先股", "境内未上市、未挂牌公司股权投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其他股权类投资", "境内未上市、未挂牌公司股权投资");
        // 上市公司定向增发投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("上市公司定向增发股票投资", "上市公司定向增发投资");
        // 新三板投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("新三板挂牌企业投资", "新三板投资");
        // 境内证券投资规模
        ASSET_ALLOCATION_TYPE_MAPPER.put("结算备付金", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("存出保证金", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("股票投资", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("债券投资", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其中:银行间市场债券", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其中:利率债", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其中:信用债", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("资产支持证券", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("基金投资(公募基金)", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其中:货币基金", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("期货及衍生品交易保证金", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("买入返售金融资产", "境内证券投资规模");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其他证券类标的", "境内证券投资规模");
        // 资管计划投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("商业银行理财产品投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("信托计划投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("基金公司及其子公司资产管理计划投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("保险资产管理计划投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("证券公司及其子公司资产管理计划投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("期货公司及其子公司资产管理计划投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("私募基金产品投资", "资管计划投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("未在协会备案的合伙企业份额", "资管计划投资");
        // 另类投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("另类投资", "另类投资");
        // 境内债权类投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("银行委托贷款规模", "境内债权类投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("信托贷款", "境内债权类投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("应收账款投资", "境内债权类投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("各类受(收)益权投资", "境内债权类投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("票据(承兑汇票等)投资", "境内债权类投资");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其他债权投资", "境内债权类投资");
        // 境外投资
        ASSET_ALLOCATION_TYPE_MAPPER.put("境外投资", "境外投资");
        // 其他资产
        ASSET_ALLOCATION_TYPE_MAPPER.put("其他资产", "其他资产");
        // 基金负债情况
        ASSET_ALLOCATION_TYPE_MAPPER.put("债券回购总额", "基金负债情况");
        ASSET_ALLOCATION_TYPE_MAPPER.put("融资、融券总额", "基金负债情况");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其中:融券总额", "基金负债情况");
        ASSET_ALLOCATION_TYPE_MAPPER.put("银行借款总额", "基金负债情况");
        ASSET_ALLOCATION_TYPE_MAPPER.put("其他融资总额", "基金负债情况");
    }

    /**
     * 构建基金资产配置信息
     *
     * @param fileId      文件id
     * @param detail      资产类型（清理过特殊字符的字符串）
     * @param mvAndRemark 市值或备注
     * @param dtos        结果数据
     */
    public static void buildAssetAllocation(Integer fileId, String detail, String mvAndRemark, List<ReportAssetAllocationDTO> dtos) {
        if (StrUtil.isBlank(mvAndRemark)) {
            return;
        }
        // 资产大类
        String assetType = ReportParseUtils.ASSET_ALLOCATION_TYPE_MAPPER.get(detail);
        if (StrUtil.contains(mvAndRemark, "#")) {
            // 有#表示有备注，而且可能有多个，多个用分号分隔的.
            List<String> marketValueAndRemarks = StrUtil.split(mvAndRemark, ";");
            for (String mr : marketValueAndRemarks) {
                if (StrUtil.isBlank(mr)) {
                    continue;
                }
                List<String> mrs = StrUtil.split(mr, "#");
                ReportAssetAllocationDTO dto = new ReportAssetAllocationDTO(fileId);
                dto.setAssetType(assetType);
                dto.setAssetDetails(detail);
                dto.setMarketValue(mrs.get(1));
                dto.setRemark(mrs.get(0));
                dtos.add(dto);
            }
        } else {
            ReportAssetAllocationDTO dto = new ReportAssetAllocationDTO(fileId);
            dto.setAssetType(assetType);
            dto.setAssetDetails(detail);
            dto.setMarketValue(mvAndRemark);
            dtos.add(dto);
        }
    }

    /**
     * 数据清洗，替换圆括号，包含中文或英文的圆括号
     *
     * @param value /
     * @return /
     */
    public static String cleaningValue(Object value) {
        return cleaningValue(value, true);
    }

    /**
     * 数据清洗，替换圆括号，包含中文或英文的圆括号
     *
     * @param value     /
     * @param replaceEn /
     * @return /
     */
    public static String cleaningValue(Object value, boolean replaceEn) {
        return cleaningValue(value, true, replaceEn);
    }

    /**
     * 数据简单清洗，并全部转为字符串类型
     *
     * @param value              待清洗的数据
     * @param replaceParentheses 是否替换圆括号
     * @return /
     */
    public static String cleaningValue(Object value, boolean replaceParentheses, boolean replaceEn) {
        String fieldValue = StrUtil.toStringOrNull(value);
        if (!StrUtil.isNullOrUndefined(fieldValue)) {
            // 特殊字符替换，空格替换为空字符
            fieldValue = fieldValue
                    .replaceAll("\r", StrUtil.EMPTY)
                    .replaceAll("；", ";")
                    .replaceAll("：", ":")
                    .replaceAll(" ", StrUtil.EMPTY)
                    .replaceAll(":", StrUtil.EMPTY);
            if (replaceEn) {
                fieldValue = fieldValue.replaceAll("[a-zA-Z]", StrUtil.EMPTY);
            }
            if (replaceParentheses) {
                // 正则表达式匹配中文括号及其内容，并替换为空字符串
                fieldValue = Pattern.compile("\\([^)]*\\)").matcher(fieldValue).replaceAll(StrUtil.EMPTY);
            }
        }
        // 如果仅有 “-” 该字段值为null
        if (Objects.equals("-", fieldValue)) {
            fieldValue = null;
        }
        return StrUtil.isBlank(fieldValue) ? null : fieldValue;
    }

    /**
     * 匹配分级基金名称（并且把母基金追加到第一行）
     *
     * @param text 文本内容
     * @return /
     */
    public static List<String> matchTieredFund(String text) {
        List<String> matches = ListUtil.list(false);
        if (StrUtil.isBlank(text)) {
            matches.add("母基金");
            return matches;
        }
        // 使用正则表达式查找匹配项
        Pattern pattern = Pattern.compile("[A-F]级|基金[A-F]");
        Matcher matcher = pattern.matcher(text);
        // 收集所有匹配项
        while (matcher.find()) {
            matches.add(matcher.group());
        }
        // 提取字母并按字母顺序排序
        List<String> levels = matches.stream()
                .map(s -> s.replaceAll("[^A-F]", ""))
                .distinct()
                .sorted()
                .map(letter -> letter + "级")
                .collect(Collectors.toList());
        levels.add(0, "母基金");
        return levels;
    }

    /**
     * 分级基金名称
     *
     * @param text 文本内容
     * @return /
     */
    public static String matchFundLevel(String text) {
        // 使用正则表达式查找匹配项
        Pattern pattern = Pattern.compile("[A-F]级|基金[A-F]");
        Matcher matcher = pattern.matcher(text);
        String result = null;
        while (matcher.find()) {
            result = matcher.group();
        }
        if (StrUtil.isBlank(result)) {
            result = "母基金";
        } else {
            result = result.replaceAll("[^A-F]", "") + "级";
        }
        return result;
    }

    /**
     * 匹配基金代码
     *
     * @param text 字符串
     * @return /
     */
    public static String matchFundCode(String text) {
        if (StrUtil.isBlank(text)) {
            return null;
        }
        Pattern pattern = Pattern.compile("S[A-Z0-9]{5}");
        Matcher matcher = pattern.matcher(text);
        if (matcher.find()) {
            return matcher.group();
        }
        return null;
    }

    /**
     * 匹配报告日期
     *
     * @param text 文本内容
     * @return 报告日期
     */
    public static String matchReportDate(String text) {
        return Optional.ofNullable(text)
                .flatMap(str -> PATTERNS.entrySet().stream()
                        .map(entry -> {
                            Matcher matcher = entry.getKey().matcher(str);
                            return matcher.find() ? entry.getValue().apply(matcher) : null;
                        })
                        .filter(result -> result != null)
                        .findFirst())
                .orElse(null);
    }

    /**
     * 匹配报告类型，如“季度”、“年度”
     *
     * @param emailTYpe 邮件类型
     * @param string    输入字符串
     * @return 匹配到的报告类型子字符串，如果没有匹配到则返回null
     */
    public static ReportType matchReportType(Integer emailTYpe, String string) {
        if (emailTYpe == null) {
            return null;
        }
        ReportType reportType = null;
        if (Objects.equals(EmailTypeConst.REPORT_EMAIL_TYPE, emailTYpe)) {
            // 类型识别---先识别季度报告，没有季度再识别年度报告，最后识别月报
            Pattern pattern = Pattern.compile("(\\d{1,2})月");  // xxxx_xx月
            Matcher matcher = pattern.matcher(string);
            if (StrUtil.containsAny(string, ReportType.QUARTERLY.getPatterns())) {
                reportType = ReportType.QUARTERLY;
            } else if (StrUtil.containsAny(string, ReportType.ANNUALLY.getPatterns())) {
                reportType = ReportType.ANNUALLY;
            } else if (StrUtil.containsAny(string, ReportType.MONTHLY.getPatterns())) {
                reportType = ReportType.MONTHLY;
            } else if (matcher.find()) {
                // 特殊的月报（当季度->年度->月度报告无法识别时，如果包含“\\d{1,2}月”就说明也是月报）
                reportType = ReportType.MONTHLY;
            }
            return reportType;
        }
        //
        if (StrUtil.containsAny(string, ReportType.LETTER.getPatterns())) {
            reportType = ReportType.LETTER;
        } else if (StrUtil.containsAny(string, ReportType.WEEKLY.getPatterns())) {
            reportType = ReportType.WEEKLY;
        } else if (StrUtil.containsAny(string, ReportType.OTHER.getPatterns())) {
            reportType = ReportType.OTHER;
        }
        return reportType;
    }

    private static int getLastDayOfMonth(int year, int month) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month - 1); // Calendar.MONTH 是从0开始的
        return calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
    }

    private static String padZero(String number) {
        return String.format("%02d", Integer.parseInt(number));
    }

    /**
     * 计算指定月份的最后一天
     */
    private static String formatMonthEnd(int year, int month) {
        try {
            YearMonth ym = YearMonth.of(year, month);
            return String.format("%d-%02d-%02d", year, month, ym.lengthOfMonth());
        } catch (Exception e) { // 处理非法月份（如month=13）
            return null;
        }
    }

    public static void main(String[] args) throws IOException, ReportParseException {
        String s = """
                =?utf-8?b?5oGS5aSp5Y2D6LGh5LqM5pyf56eB5Yuf6K+B5Yi45oqV6LWE5Z+66YeRLeaBkg==?=
                 天千象二期私募证券投资基金2025年4月月度报告-20250512.pdf
                """;
        String s1 = MimeUtility.decodeText(s);
        System.out.println(s1);
    }
}
